==> twc-0.1.0.tar.gz: twc-0.1.0/src/twc/widgets/completions.py <==
# Copyright (C) 2019 Michał Góral.
#
# This file is part of TWC
#
# TWC is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# TWC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TWC. If not, see <http://www.gnu.org/licenses/>.


def keywords():
    times = [
        'ascension', 'easter', 'eastermonday', 'eocw', 'eoww', 'eom', 'eoq',
        'eow', 'eoy', 'fri', 'goodfriday', 'later', 'midsommar',
        'midsommarafton', 'mon', 'pentecost', 'sat', 'socw',
        'som', 'someday', 'soq', 'sow', 'soww', 'soy', 'sun', 'thu',
        'today', 'tomorrow', 'tue', 'wed'
    ]

    completions = [
        'depends:', 'description:', 'due:', 'end:', 'entry:',
        'limit:', 'priority:', 'project:', 'recur:', 'scheduled:',
        'start:', 'status:', 'until:', 'wait:']

    for spec in ['due', 'until', 'wait']:
        completions.extend(['{}:{}'.format(spec, time) for time in times])

    return completions


def udas(tw):
    completions = []
    # add UDAs
    for uda in tw.execute_command(['_udas']):
        completions.append('{}:'.format(uda))
    return completions


def tags(tw):
    completions = set()
    for comma_sep_tags in tw.execute_command(['_unique', 'tags']):
        taglist = comma_sep_tags.split(',')
        for tag in taglist:
            completions.add('+{}'.format(tag))
    return completions


def annotations(task):
    return set(ann['description'] for ann in task.t['annotations'])


def task_add(tw):
    completions = keywords()
    completions.extend(udas(tw))
    completions.extend(tags(tw))
    completions.sort()
    return completions


def task_modify(task, tw):
    completions = keywords()
    completions.extend(udas(tw))

    # tags
    missing_tags = set()
    for comma_sep_tags in tw.execute_command(['_unique', 'tags']):
        taglist = comma_sep_tags.split(',')
        for tag in taglist:
            if tag not in task.t['tags']:
                missing_tags.add('+{}'.format(tag))
    completions.extend(missing_tags)
    completions.extend('-{}'.format(tag) for tag in task.t['tags'])
    completions.append(task.t['description'])
    completions.sort()
    return completions

==> twc-0.3.1.tar.gz: twc-0.3.1/src/twc/widgets/completions.py <==
# Copyright (C) 2019 Michał Góral.
#
# This file is part of TWC
#
# TWC is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# TWC is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TWC. If not, see <http://www.gnu.org/licenses/>.

from twc.twutils import execute_command


def keywords():
    times = [
        'ascension', 'easter', 'eastermonday', 'eocw', 'eoww', 'eom', 'eoq',
        'eow', 'eoy', 'fri', 'goodfriday', 'later', 'midsommar',
        'midsommarafton', 'mon', 'pentecost', 'sat', 'socw',
        'som', 'someday', 'soq', 'sow', 'soww', 'soy', 'sun', 'thu',
        'today', 'tomorrow', 'tue', 'wed'
    ]

    completions = [
        'depends:', 'description:', 'due:', 'end:', 'entry:',
        'limit:', 'priority:', 'project:', 'recur:', 'scheduled:',
        'start:', 'status:', 'until:', 'wait:']

    for spec in ['due', 'until', 'wait']:
        completions.extend(['{}:{}'.format(spec, time) for time in times])

    return completions


def udas(tw):
    completions = []
    # add UDAs
    for uda in execute_command(tw, '_udas').verified_out():
        completions.append('{}:'.format(uda))
    return completions


def tags(tw):
    completions = set()
    for comma_sep_tags in execute_command(tw, '_unique', 'tags').verified_out():
        taglist = comma_sep_tags.split(',')
        for tag in taglist:
            completions.add('+{}'.format(tag))
    return completions


def annotations(task):
    return set(ann['description'] for ann in task.t['annotations'])


def task_add(tw):
    completions = keywords()
    completions.extend(udas(tw))
    completions.extend(tags(tw))
    completions.sort()
    return completions


def task_modify(task, tw):
    completions = keywords()
    completions.extend(udas(tw))

    # tags
    missing_tags = set()
    for comma_sep_tags in execute_command(tw, '_unique', 'tags').verified_out():
        taglist = comma_sep_tags.split(',')
        for tag in taglist:
            if tag not in task.t['tags']:
                missing_tags.add('+{}'.format(tag))
    completions.extend(missing_tags)
    completions.extend('-{}'.format(tag) for tag in task.t['tags'])
    completions.append(task.t['description'])
    completions.sort()
    return completions
